/* vim: set foldmethod=marker : */

#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdint.h>
#include "cc2500.h"
#include "cc2500_cfg.h"

#define CC2500_BUFSIZE 64

#define CC2500_SELECT { PORTB &= ~_BV(4); }
#define CC2500_DESELECT { PORTB |= _BV(4); }
#define CC2500_WAIT { while (PINB & _BV(6)); }

#define CC2500_BURST 0x40
#define CC2500_READ  0x80
#define CC2500_ONEBYTE  0x20
#define CC2500_COMMAND  0x10

static volatile uint8_t timeout = 0;


static uint8_t spi_move(uint8_t data)/*{{{*/
{
	SPDR = data;
	while (!(SPSR & _BV(SPIF)));
	return SPDR;
}/*}}}*/

uint8_t cc2500_status(uint8_t tx)/*{{{*/
{
	uint8_t ret;
	CC2500_SELECT;
	CC2500_WAIT;
	ret = spi_move(tx ? 0x00 : 0x80);
	CC2500_DESELECT;
	return ret;
}/*}}}*/

void cc2500_por(void)/*{{{*/
{
	CC2500_SELECT;
	_delay_us(1);
	CC2500_DESELECT;
	_delay_us(40);
	CC2500_SELECT;
	SPDR = 0x30;
	while (!(SPSR & _BV(SPIF)));
	CC2500_DESELECT;
}/*}}}*/

static uint8_t cc2500_move(uint8_t address, uint8_t mask, uint8_t n, uint8_t *buf)/*{{{*/
{
	uint8_t ret = n;
	address &= 0x3F;
	if ((!(mask & CC2500_COMMAND)) && (address > 0x2F)) {
		mask |= CC2500_BURST;
	}
	if (mask & CC2500_ONEBYTE) {
		buf = &ret;
		n = 1;
	}
	mask &= (CC2500_READ | CC2500_BURST);
	address |= mask;
	CC2500_SELECT;
	CC2500_WAIT;
	spi_move(address);
	for ( ; n ; n-- ) {
		if (mask & CC2500_READ) {
			*buf = spi_move(0x00);
		} else {
			spi_move(*buf);
		}
		buf++;
	}
	CC2500_DESELECT;
	return ret;
}/*}}}*/

uint8_t cc2500_read(uint8_t address)/*{{{*/
{
	return cc2500_move(address, CC2500_ONEBYTE | CC2500_READ, 0, 0);
}/*}}}*/

void cc2500_read_burst(uint8_t address, uint8_t n, uint8_t *buf)/*{{{*/
{
	cc2500_move(address, CC2500_READ | CC2500_BURST, n, buf);
}/*}}}*/

void cc2500_write(uint8_t address, uint8_t data)/*{{{*/
{
	cc2500_move(address, CC2500_ONEBYTE, data, 0);
}/*}}}*/

void cc2500_write_burst(uint8_t address, uint8_t n, uint8_t *buf)/*{{{*/
{
	cc2500_move(address, CC2500_BURST, n, buf);
}/*}}}*/

void cc2500_command(uint8_t command)/*{{{*/
{
	cc2500_move(command, CC2500_COMMAND, 0, 0);
}/*}}}*/

void cc2500_init(void)/*{{{*/
{
	int i;
	for (i = 0; i < sizeof(cc2500_cfg); i+=2) {
		cc2500_write(pgm_read_byte(cc2500_cfg + i), pgm_read_byte(cc2500_cfg + i +1));
	}
}/*}}}*/

