#include <hm2007.h>
#include <binary.h>
#include <avr/io.h>
#include <util/delay.h>

#define HM_OUTPUT {HM_K_DDR |= (0x0F << HM_K_BIT); }
#define HM_INPUT {HM_K_DDR &= ~(0x0F << HM_K_BIT); }
#define HM_WRITE(c) {HM_K_PORT = (HM_K_PORT & ~(0x0F << HM_K_BIT)) | ((c & 0x0F) << HM_K_BIT);}

static uint8_t wlen = 0;

void hm_wlen(uint8_t len)
{
	if (len) {
		HM_WL_PORT |= (1 << HM_WL_BIT);
	} else {
		HM_WL_PORT &= ~(1 << HM_WL_BIT);
	}
	wlen = len;
}

static uint8_t hm_read(void)
{
	return (HM_K_PIN & (0x0F << HM_K_BIT)) >> HM_K_BIT;
}

static uint8_t hm_status(void)
{
	uint8_t ret = hm_read() & 0x03;
	return ret;
}

uint8_t hm_ready(void)
{
	return hm_status() == 2;
}


static void hm_s(uint8_t x)
{
	HM_S_PORT = (HM_S_PORT & ~(7 << HM_S_BIT)) | (x & (7 << HM_S_BIT));
	_delay_us(100);
}

static uint8_t hm_read_byte_nibble(uint8_t status)
{
	uint8_t ret;
	while (hm_status() != status);
	hm_s(B8(011));
	ret = hm_read();
	hm_s(B8(101));
	hm_s(B8(001));
	hm_s(B8(000));
	hm_s(B8(010));
	return ret;
}

uint8_t hm_read_byte(void)
{
	return hm_read_byte_nibble(3) | (hm_read_byte_nibble(0) << 4);
}

static void hm_command(uint8_t command)
{
	hm_s(B8(010));
	while (hm_status() != 2);
	hm_s(B8(000));
	HM_OUTPUT;
	HM_WRITE(command & 0x0F);
	hm_s(B8(100));
	HM_INPUT;
	hm_s(B8(010));
}

static void hm_write_byte_nibble(uint8_t status, uint8_t nibble)
{
	while (hm_status() != status);
	hm_s(B8(000));
	HM_OUTPUT;
	HM_WRITE(nibble & 0x0F);
	hm_s(B8(100));
	HM_INPUT;
	hm_s(B8(010));
}

void hm_write_byte(uint8_t byte)
{
	hm_write_byte_nibble(3, byte & 0x0F);
	hm_write_byte_nibble(0, byte >> 4);
}

void hm_init(void)
{
	HM_S_DDR |= (0x07 << HM_S_BIT);
	HM_K_DDR &= ~(0x0F << HM_K_BIT);

	HM_WL_PORT &= ~(1 << HM_WL_BIT);
	HM_WL_DDR |= (1 << HM_WL_BIT);

}

void hm_recog(void)
{
	hm_command(1);
	while (hm_status() != 1);
}

uint16_t hm_result(void)
{
	uint8_t pattern, score;
	hm_command(4);
	HM_WL_PORT &= ~(1 << HM_WL_BIT);
	pattern = hm_read_byte();
	score = hm_read_byte();
	if (wlen) {
		HM_WL_PORT |= (1 << HM_WL_BIT);
	}
	return pattern | (score << 8);
}

void hm_train(uint8_t pattern)
{
	hm_command(2);
	hm_write_byte(pattern);
}

void hm_reset(void)
{
	hm_command(7);
	while (hm_status() != 2);
}

uint16_t hm_pattern_size(uint8_t pattern)
{
	hm_command(5);
	hm_write_byte(pattern);
	return hm_read_byte() << 2;
}

void hm_download(uint16_t size, uint8_t *buf)
{
	uint16_t i;
	for (i =0; i < size; i++) {
		buf[i] = hm_read_byte();
	}
}

void hm_start_upload(uint8_t pattern, uint16_t len)
{
	hm_command(6);
	hm_write_byte(pattern);
	hm_write_byte(len >> 2);
}

