#ifndef HM2007_H
#define HM2007

#include <stdint.h>

/**
 * @mainpage
 *
 * \par   
 * 
 * \par
 *
 * HM2007 - ,      
 *  .     : 8 KiB ,  ,
 *    .     40 
 *   0.9 ,   20    1.9 .
 *       8 :
 * - 4-    (K1-K4)
 * - 3     (S1-S3)
 * -     (WLEN)
 *
 *
 * 	\par  
 *
 * 	\par
 *
 * 	  ,   ,   .
 * 	        30  240 
 * 	(  ).    
 * 	    . ,  
 * 	  .     
 * 	        
 * 	.
 */

/** @defgroup HM2007 HM2007 */
/*@{*/

/**
 * @brief         HM2007.
 *
 * \par  .
 * \par
 *
 * 	-  S1-S3        ,  
 * 	  . ,  S1   PD3,  S2     PD4,
 * 	   S3 -  PD5
 * 	-  K1-K4        ,  
 * 	  . ,  K1   PB2,  K2    
 * 	  PB3, K3 -  PB4,  K4 -   PB5.
 * 	-  WLEN      .
 * 	-  S1-S3     5 .. 10 
 */


/**
 * @name  
 */

#define HM_S_PORT PORTB 	/**<   ,     S1-S3  */
#define HM_S_DDR  DDRB		/**<       S1-S3 */
#define HM_S_BIT  0			/**<  ,     S1 */

#define HM_K_PORT PORTC 	/**<   ,     K1-K4  */
#define HM_K_DDR  DDRC		/**<      K1-K4 */
#define HM_K_PIN  PINC 	/**<   ,     K1-K4  */
#define HM_K_BIT  4			/**<  ,     K1 */

#define HM_WL_PORT PORTC 	/**<   ,     WLEN  */
#define HM_WL_DDR  DDRC		/**<       WLEN */
#define HM_WL_BIT  3			/**<  ,     WLEN */

/**
 * @name 
 */


/**
 * @brief .
 *        HM2007.
 *           .
 *    0.9 .
 * @see hm_wlen()
 */
void hm_init(void);

/**
 * @brief  .
 *    .
 */
void hm_reset(void);

/**
 * @brief   .
 * @return 1       ,
 *  0
 */
uint8_t hm_ready(void);

/**
 * @brief    .
 *    ,   ,
 *   hm_ready().   
 *    ,  hm_ready()   1.
 *        
 * hm_result().
 */
void hm_recog(void);

/**
 * @brief    .
 *    ,   ,
 *   hm_ready().   
 *    ,  hm_ready()   1.
 *
 * @param pattern     .
 *  1  20  40,     .
 * @see hm_wlen()
 * @warning      ,  
 *     (0.9  1.9 ,
 *     ),   ""
 * .
 */
void hm_train(uint8_t pattern);

/**
 * @brief  .
 *       hm_recog()
 *   .
 *
 * @return   .
 * <br>  8    
 * .
 * <br>  8      0  255.
 *    255,     . 0 - 
 * 100% .      32
 *  255.
 * @see hm_recog()
 */
uint16_t hm_result(void);

/**
 * @brief    ,   
 *   .
 *       .   
 *   ,     
 * . hm_pattern_size()    , 
 *     .    
 *     hm_download(),   
 *     ,  hm_read_byte().
 *
 * @param pattern  .
 *  1  20  40,     .
 * @return   .   4
 * @see hm_wlen()
 * @warning    ,    ,
 *   ,  "".
 */
uint16_t hm_pattern_size(uint8_t pattern);

/**
 * @brief   .
 *        
 *  hm_pattern_size()  hm_train().
 * @param len 0 - 0.9  ; 1 - 1.9 . 
 * @warning    ,    
 * hm_reset().
 */
void hm_wlen(uint8_t len);

/**
 * @brief        
 *   .
 *       hm_pattern_size(). ,
 * ,   ,     
 *       (-
 * ,  ,    ..).    
 *     hm_start_upload()  hm_write_byte().
 * @param size   ,   hm_pattern_size().
 * @param buf   .     \b size .
 */
void hm_download(uint16_t size, uint8_t *buf);

/**
 * @brief     .
 *   hm_pattern_size(),       ,
 *    hm_pattern_size().
 * @return   .
 */
uint8_t hm_read_byte(void);


/**
 *      ,   .
 *      ,   
 *   hm_pattern_size(), hm_download() / hm_read_byte().
 *      \b len   hm_write_byte().
 *
 * @param pattern  
 * @param len     .  ,
 *   hm_pattern_size()
 */
void hm_start_upload(uint8_t pattern, uint16_t len);

/**
 *      ,   .
 *      ,   
 *   hm_pattern_size(), hm_download() / hm_read_byte().
 *     hm_start_upload().
 *
 * @param byte   
 */
void hm_write_byte(uint8_t byte);

/*@}*/

#endif


